#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
PyQt5 client web

Environment: ubuntu 18.04
Author: dell author
Last edited: January 2018

"""
import os
import re
import json
import dbus
import time
import logging

from PyQt5.QtCore import Qt, QUrl, pyqtSlot, QMutex, pyqtSignal, QBasicTimer, QThread
from PyQt5 import QtWebChannel, QtWidgets, QtGui
from PyQt5.QtWidgets import QDialog, QMainWindow, QProgressBar
from PyQt5.QtWebEngineWidgets import QWebEngineView, QWebEnginePage


from DellLinuxAssistant.portal.dell_linux_assistant import (Ui_MainDialog,
                                                                     Ui_BoardDialog,
                                                                     Ui_ProductWindow)

from DellLinuxAssistant.registration.constant import Constants
from DellLinuxAssistant.portal.utils.q_object_handler import (portalSharedObject,
                                                           SystemInfoSharedObject,
                                                           RegistrationSharedObject)
import DellLinuxAssistant.registration.utils.RegistrationHelper as RegistrationHelper
from DellLinuxAssistant.registration.utils.BILRegistration import MyDellRegistration
from DellLinuxAssistant.telemetry.telemetry_common import check_version, Struct, ObjectThread
from DellLinuxAssistant.telemetry.dbus_helper import DbusHandler

UIDIR = "/usr/share/dell-linux-assistant/registration/html"

class MainDialog(QDialog):
    """
    init dialog
    """
    def __init__(self):
        QDialog.__init__(self)
        self.main_ui = Ui_MainDialog()
        self.main_ui.setupUi(self)
        self.center()
        self.setWindowFlags(Qt.FramelessWindowHint)

    def center(self):
        screen = QtWidgets.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2)

    def __del__(self):
        """clear cache"""
        self.deleteLater()

class BoardDialog(QDialog):
    """
    BoardDialog
    """
    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.child = Ui_BoardDialog()

        self.child.setupUi(self)
        self.child.pushButton.setStyleSheet("background-color:rgb(238, 238, 236);")

    def __del__(self):
        """clear cache"""
        self.deleteLater()

class WebEnginePage(QWebEnginePage):
    """
    QWebEnginePage
    """
    def __init__(self, parent=None):
        super(WebEnginePage, self).__init__(parent)

    def createWindow(self, windowType):
        """
        web view a link single
        :param windowType:
        :return:
        """
        page = QWebEnginePage(self)
        page.urlChanged.connect(self.on_url_changed)
        return page

    @pyqtSlot(QUrl)
    def on_url_changed(self, url):
        """
        web view a link slot
        :param url: a link url
        :return:
        """
        page = self.sender()
        QtGui.QDesktopServices.openUrl(url)
        page.deleteLater()

    def chooseFiles(self, mode, oldFiles, acceptedMimeTypes):
        """
        html type=file property
        :param mode: FileSelectionMode
        :param oldFiles: QStringList
        :param acceptedMimeTypes: QStringList
        :return:
        """
        return []

class WindowHandler(object):

    def __init__(self):
        self.profile = None
        self.dbus_handler = DbusHandler()

    def check_domain_url(self, host):
        """
        check client request url
        :param host: domain url
        :return:
        """
        prog = re.compile(r"^(\w+.*)((fusncloud-np.net)|(fusnwebsvc.net))")
        result = prog.search(host)

        return result

    def systemProfile(self):
        """
        system profile func
        :return:
        """
        if not self.profile:
            self.profile = self.dbus_handler.backend().get_HW_info()

        return Struct(**{"serviceTag": self.profile[0],
                "model": self.profile[3],
                "oSVersion": self.profile[10],
                "oobeDate": self.profile[12],
                "lob": self.profile[13]
                })

    def get_network(self):
        """
        network able/disable
        :return:
        """
        return self.dbus_handler.backend().get_net_work()

    def web_url_assemble(self, baseUrl, country_, language_, isRetail_):
        """
        web server url assemble
        :param baseUrl: server url
        :param country_: client system country
        :param language_: client system language
        :param isRetail_: server
        :return:
        """
        self._version = check_version("dell-linux-assistant")
        system_profile = self.systemProfile()
        selectedUri = baseUrl + \
                      Constants.SET_COUNTRY + country_.upper() + \
                      Constants.SET_LANGUAGE + language_.lower() + \
                      Constants.SET_RETAIL + str(isRetail_) + \
                      Constants.SET_SERVICE_TAG + system_profile.serviceTag + \
                      Constants.SET_MODEL + system_profile.model + \
                      Constants.SET_OOBE + system_profile.oobeDate + \
                      Constants.SET_APP_VERSION + self._version + \
                      Constants.SET_OS_EDITION + "LINUX" + \
                      Constants.SET_OS_VERSION + system_profile.oSVersion + \
                      Constants.SET_OS_LOB + system_profile.lob

        return selectedUri

class QThreadWindow(QThread):
    """
    preload thread
    """
    dbus_handler = DbusHandler()
    window_handler = WindowHandler()
    web_view_hide_show_signal = pyqtSignal(dict)

    _running = True
    _mutex = QMutex()
    ssl_verify_flag = False
    def __int__(self):
        super(QThreadWindow, self).__init__()

    def cancel_pre_load_flag(self):
        """
        cancel pre load
        :return:
        """
        try:
            self._mutex.lock()
            self._running = False
        finally:
            self._mutex.unlock()

    def set_ssl_verify_flag(self):
        """
        set ssl verity flag True
        :return:
        """
        try:
            self._mutex.lock()
            self.ssl_verify_flag = True
        finally:
            self._mutex.unlock()

    def check_network_status_freq(self, n=1):
        """
        check network 3n/3s freq
        :return:
        """
        count = 3
        net_status = False
        while(count>0 and self._running):
            network = self.window_handler.get_network()
            if network == 'ABLED':
                net_status = True
                break
            count-=1
            time.sleep(1)

        if net_status or not self._running:
            return net_status
        else:
            # time wait n minutes frequency
            if n == 8:
                return net_status

            t = n * 60
            time.sleep(t)
            n+=1
            return self.check_network_status_freq(n)

    def server_web_url(self, environment_, country_, language_, isRetail_):
        """
        register user
        :param environment: the development environment
        :param country: Event handler for the country combo box selection
        :param language: Event handler for the language combo box selection
        :param isRetail: The event of IsRetailButton
        :return: success or fail
        """
        skip_domain_cert_chk = False
        if str(environment_).lower() == "qa":
            baseUrl_key = 'QA_BASE_URL'
        elif str(environment_).lower() == "staging":
            baseUrl_key = 'STAGING_BASE_URL'
        elif str(environment_).lower() == "production":
            baseUrl_key = 'PRODUCTION_BASE_URL'
        else:
            baseUrl_key = 'DEV_BASE_URL'
            skip_domain_cert_chk = True

        if not self.check_network_status_freq():
            return

        _input = str(json.dumps({'url_name': baseUrl_key}))
        baseUrl = self.dbus_handler.backend().get_library_info("GetRegistrationUrl", _input)
        if not baseUrl:
            return

        # for testing purpose, will skip domain check and ssl cert verify
        if not skip_domain_cert_chk and self._running:

            if not self.window_handler.check_domain_url(baseUrl):
                self.dbus_handler.dbus_exception_handler(dbus.DBusException("client load web url error: {} "
                                                                            "please confirm!".format(baseUrl)))
                return

            _input = json.dumps({'environment_': environment_.upper()})
            _output = self.dbus_handler.backend().get_library_info('VerifySSLFingerprint', _input)
            if _output == 'unknow':
                return
            else:
                self.set_ssl_verify_flag()

        if self._running:
            selectedUri = self.window_handler.web_url_assemble(baseUrl, country_, language_, isRetail_)
            return selectedUri
        return

    def run(self):
        """
        preload main thread run start
        :return:
        """
        country = RegistrationHelper.get_os_country()
        language = RegistrationHelper.get_os_language()
        selectedUri = self.server_web_url('dev', country, language, False)

        if selectedUri:
            self.web_view_hide_show_signal.emit({'selectedUri':selectedUri, 'version': check_version("dell-linux-assistant")})
        else:
            logging.error("[PRE LOAD WEBVIEW FAIL]")

class ProductWindow(QMainWindow):
    """
    ProductWindow
    """
    web_load_fail_signal = pyqtSignal()

    def __init__(self):
        QMainWindow.__init__(self)
        self.child = Ui_ProductWindow()
        self.child.setupUi(self)

        self.view = QWebEngineView()
        self.preload_view = QWebEngineView()
        self.dbus_handler = DbusHandler()
        self.window_handler = WindowHandler()
        self.BILRegistration = MyDellRegistration()

        self.profile = None

        self.main_browser(FirstLoad=True)

        self.setCentralWidget(self.view)
        self.center()

        self.pbar = QProgressBar(self)
        self.pbar.setGeometry(200, 700, 700, 25)
        self.timer = QBasicTimer()
        self.pbar.hide()

        self._running = True
        self._mutex = QMutex()
        self._load_start = False
        self.web_load_fail_signal.connect(self.web_load_page)
        self.view.loadStarted.connect(self.loadStarted)
        self.view.loadFinished.connect(self.loadFinished)
        self.view.loadProgress.connect(self.loadProgress)

    def center(self):
        """
        qt center display
        :return:
        """
        screen = QtWidgets.QDesktopWidget().screenGeometry()
        size = self.geometry()
        self.move((screen.width() - size.width()) / 2, (screen.height() - size.height()) / 2)

    def main_browser(self, FirstLoad=False):
        """
        main browser
        :param reload: load view fail, reload callback registration page
        :return:
        """
        self.dbus_handler.LogComplexEvent('FRONTEND', 'frontend UI is running')

        self._load_start = False

        UIDIR = "/usr/share/dell-linux-assistant/portal/html"
        main_url = "file://" + os.path.join(UIDIR, 'portal.html')

        # associate page with view
        self.page = WebEnginePage()
        self.webchannel = QtWebChannel.QWebChannel(self.page)
        self.portalSharedObject = portalSharedObject(page=self.page,
                                                           ProductWindow=self,
                                                           DbusHandler=self.dbus_handler)
        self.SystemInfoSharedObject = SystemInfoSharedObject(page=self.page, DbusHandler=self.dbus_handler)

        self.webchannel.registerObjects({"BoardObj": self.portalSharedObject,
                                         "SystemObj": self.SystemInfoSharedObject
                                        })
        self.page.setWebChannel(self.webchannel)
        self.page.load(QUrl(main_url))

        self.view.setPage(self.page)
        self.view.setContextMenuPolicy(Qt.NoContextMenu)

        engine_profile = self.page.profile()
        engine_profile.clearHttpCache()

        # check app first load and check user register success
        if FirstLoad and not self.BILRegistration.GetRegistrationEvent():
            self.webview_pre_load()

    def server_web_url(self, environment_, country_, language_, isRetail_):
        """
        register user
        :param environment: the development environment
        :param country: Event handler for the country combobox selection
        :param language: Event handler for the language combo box selection
        :param isRetail: The event of IsRetailButton
        :return: success or fail
        """
        skip_domain_cert_chk = False
        if str(environment_).lower() == "qa":
            baseUrl_key = 'QA_BASE_URL'
        elif str(environment_).lower() == "staging":
            baseUrl_key = 'STAGING_BASE_URL'
        elif str(environment_).lower() == "production":
            baseUrl_key = 'PRODUCTION_BASE_URL'
        else:
            baseUrl_key = 'DEV_BASE_URL'
            skip_domain_cert_chk = True

        network = self.window_handler.get_network()
        if network == 'DISABLED':
            self.dbus_handler.dbus_massage('No network connection. Can not retrive registration page. ')
            return

        _input = str(json.dumps({'url_name': baseUrl_key}))
        baseUrl = self.dbus_handler.backend().get_library_info("GetRegistrationUrl", _input)
        if not baseUrl:
            self.dbus_handler.dbus_exception_handler(dbus.DBusException("client web url error: {} "
                                                                        "please confirm!".format(baseUrl)))
            return

        self._version = check_version("dell-linux-assistant")
        # for testing purpose, will skip domain check and ssl cert verify
        if not skip_domain_cert_chk:

            if not self.window_handler.check_domain_url(baseUrl):
                self.dbus_handler.dbus_exception_handler(dbus.DBusException("client load web url error: {} "
                                                                            "please confirm!".format(baseUrl)))
                return

            # check preload whether verify url ssl
            if not(hasattr(self, 'qThread') and self.qThread.ssl_verify_flag):
                _input = json.dumps({'environment_': environment_.upper()})

                _output = self.dbus_handler.backend().get_library_info('VerifySSLFingerprint', _input)
                if _output == 'unknow':
                    self.dbus_handler.dbus_exception_handler(dbus.DBusException("check server ssl error: {} "
                                                                                "please confirm!".format(baseUrl)))
                    return

        selectedUri = self.window_handler.web_url_assemble(baseUrl, country_, language_, isRetail_)
        return selectedUri


    def LaunchWebView(self,selectedEnvironment, selectedCountry, selectedLanguage, selectedRetailOption):

        selectedUri = self.server_web_url(selectedEnvironment, selectedCountry, selectedLanguage, selectedRetailOption)
        if not selectedUri:
            return

        self.pbar.show()
        self._load_start = True

        # associate page with view
        self.page = WebEnginePage()

        engineProfile = self.page.profile()
        engineProfile.clearHttpCache()

        self.serverchannel = QtWebChannel.QWebChannel(self.page)
        self.sharedObj = RegistrationSharedObject(ProductWindow=self,
                                                  clientAppVersion=self._version,
                                                  DbusHandler=self.dbus_handler)
        self.serverchannel.registerObject("sharedObj", self.sharedObj)
        self.page.setWebChannel(self.serverchannel)
        self.page.setBackgroundColor(QtGui.QColor(0, 125, 184))
        self.page.load(QUrl(selectedUri))
        if hasattr(self, 'qThread'):
            self.qThread.cancel_pre_load_flag()

        self.dbus_handler.LogComplexEvent('FRONTEND', 'frontend UI is running')

        self.view.setPage(self.page)
        self.view.setContextMenuPolicy(Qt.NoContextMenu)


    def webview_pre_load(self):
        """
        preload thread start
        :return:
        """
        self.qThread = QThreadWindow()
        self.qThread.web_view_hide_show_signal.connect(self.webview_pre_show)

        self.qThread.start()


    def webview_pre_show(self, info):
        """
        preload webview load slot
        :param info:
        :return:
        """
        self.preload_view.hide()
        # add flag because clear preload_view cache not allow repeat
        self.preload_webview_success_flag = True

        # associate page with view
        self.page = QWebEnginePage()

        engineProfile = self.page.profile()
        engineProfile.clearHttpCache()

        self.serverchannel = QtWebChannel.QWebChannel(self.page)
        self._version = info['version']
        self.sharedObj = RegistrationSharedObject(ProductWindow=self,
                                                  clientAppVersion=self._version,
                                                  DbusHandler=self.dbus_handler)
        self.serverchannel.registerObject("sharedObj", self.sharedObj)
        self.page.setWebChannel(self.serverchannel)
        self.page.setBackgroundColor(QtGui.QColor(0, 125, 184))
        self.page.load(QUrl(info['selectedUri']))

        self.preload_view.setPage(self.page)
        self.preload_view.setContextMenuPolicy(Qt.NoContextMenu)


    def timerEvent(self, event):
        """
        load web view progress bar task
        :param event:
        :return:
        :return:
        """
        if self.process >= 100:
            self.timer.stop()
            return

        self.pbar.setValue(self.process)


    def web_load_page(self):
        """
        load web view fail, reback registration page
        :return:
        """
        reply = self.dbus_handler.dbus_question_massage("The network is loading slowly. If you want cancel load? ")
        if reply:
            try:
                # reload register url
                self.timer.stop()
                self.main_browser()
            except Exception as e:
                logging.error("[Opening System Registration Page]ERR:{}".format(str(e)))
        else:
            if self.process != 100:
                self.FollowStatusCheck()
                self.checkAsync(30)

    @ObjectThread(result=False)
    def checkAsync(self, interval):
        """
        check load web wether timeout
        :param interval: interval time
        :return:
        """
        flag = False
        now = int(time.time())
        while(self.running()):
            cur_time = int(time.time())
            if now+interval < cur_time:
                self.CancelFollowStatusCheck()
                flag = True
            time.sleep(1)

        if flag:
            self.web_load_fail_signal.emit()


    def CancelFollowStatusCheck(self):
        """
        cancel follow status
        :return:
        """
        try:
            self._mutex.lock()
            self._running = False
        finally:
            self._mutex.unlock()

    def FollowStatusCheck(self):
        """
        cancel follow status
        :return:
        """
        try:
            self._mutex.lock()
            self._running = True
        finally:
            self._mutex.unlock()

    def running(self):
        """
        get running flag
        :return:
        """
        try:
            self._mutex.lock()
            return self._running
        finally:
            self._mutex.unlock()

    def loadStarted(self):
        """
        web view load start signal
        """
        if self._load_start:
            self.timer.start(100, self)

            self.CancelFollowStatusCheck()
            self.FollowStatusCheck()
            self.checkAsync(30)


    def loadProgress(self, process):
        """
        load web view process
        :param process:
        :return:
        """
        try:
            self._mutex.lock()
            self.process = process
        finally:
            self._mutex.unlock()


    def loadFinished(self, bool):
        """
        web page load single
        :param bool: true: success, false: fail
        :return:
        """
        self.CancelFollowStatusCheck()

        if not bool:
            self.dbus_handler.dbus_massage("Server error, Please wait a moment request! ")
            logging.debug('web url failed! ', Constants.NAVIGATION_FAILED)

            try:
                # register url
                self.main_browser()
            except Exception as e:
                logging.error("[Opening System Registration Page]ERR:{}".format(str(e)))
        else:
            if hasattr(self, 'preload_webview_success_flag') and self.preload_webview_success_flag:
                self.preload_webview_success_flag = False
                self.preload_view.deleteLater()

        self.pbar.hide()
        self.pbar.setValue(0)
        logging.debug('web url success! ', Constants.NAVIGATION_COMPLETED)
        return True

    def switch_broswer(self, selectedEnvironment, selectedCountry, selectedLanguage, selectedRetailOption):
        """
        :param selectedEnvironment: the development environment
        :param selectedCountry: Event handler for the country combobox selection
        :param selectedLanguage: Event handler for the language combo box selection
        :param selectedRetailOption: The event of IsRetailButton
        :return:
        """
        self.LaunchWebView(selectedEnvironment, selectedCountry, selectedLanguage, selectedRetailOption)

    def js_callback(self, version):
        """
        server web js callback, display app web version
        :param version: web version
        :return:
        """
        self.WebAppVersion = "Web App " + version
        self.page.runJavaScript("var version = document.getElementById('version-section');"
                                "var version_val = document.getElementById('client-version').innerHTML;"
                                "if(version_val==''||version_val==null){"
                                "   document.getElementById('client-version').innerHTML='&nbsp&nbsp&nbsp&nbsp%s ';"
                                "   $('#version-section').show();"
                                "}"
                                "else{"
                                "   version.style.setProperty('width', '250px');"
                                "   document.getElementById('client-version').innerHTML=version_val + ' &nbsp&nbsp&nbsp&nbsp%s '; "
                                "};" % (self.WebAppVersion, self.WebAppVersion))

    def GetWebAppVersion(self):
        """
        get server web js version
        :return:
        """
        try:
            self.page.runJavaScript('getWebAppVersion();', self.js_callback)
        except Exception:
            self.WebAppVersion = 'Missing'
            logging.error("RegistrationPage Unable to call getWebAppVersion method.")


    def __del__(self):
        """clear cache"""
        self.deleteLater()


