#!/usr/bin/env python3
# coding: utf-8

#!/usr/bin/python3
# -*- coding: utf-8 -*-

"""
dbus handler

author: Tina Wang
last edited: Feb 2019
"""

import os
import sys
import dbus
import json
import time
import signal
import logging

from PyQt5 import QtCore
from PyQt5.QtWidgets import QMessageBox

from DellLinuxAssistant.telemetry.profile_helper import conf
from DellLinuxAssistant.telemetry.file_helper import FileHelper
from DellLinuxAssistant.telemetry.telemetry_common import dbus_sync_call_signal_wrapper
from DellLinuxAssistant.telemetry.dbus_backend import (DBUS_BUS_NAME,
                                                 DBUS_INTERFACE_NAME,
                                                 PermissionDeniedByPolicy)

from gettext import gettext as _


class DbusHandler(object):
    """DBUS handler"""
    _dbus_iface = None

    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)

    @staticmethod
    def dbus_exception_msg(msg):
        if msg.get_dbus_name() == 'org.freedesktop.DBus.Error.FileNotFound':
            text = _("Cannot connect to dbus")
        elif msg.get_dbus_name() == PermissionDeniedByPolicy._dbus_error_name:
            text = _("Permission Denied")
        elif msg.get_dbus_name() == 'org.freedesktop.DBus.Error.NoReply':
            text = _("The message bus security policy blocked the reply, the reply timeout expired, "
                     "please confirm server or restart server")
        else:
            text = msg.get_dbus_message()
        return text

    @staticmethod
    def cleanup_backend():
        """cleans up the backend process"""
        try:
            while 1:
                os.kill(os.getpid(), signal.SIGTERM)
                time.sleep(0.1)
        except OSError as err:
            err = str(err)
            if err.find('No such process') > 0:
                logging.warning("PID{} is killed".format(str(os.getpid())))
            else:
                logging.error('[DBUS HELPER]: ', str(err))
                sys.exit(1)

    def dbus_massage(self, msg):

        message = QMessageBox()
        message.setWindowFlag(QtCore.Qt.WindowStaysOnTopHint)
        message.setIcon(QMessageBox.Warning)

        message.setStyleSheet("background-color:rgb(238, 238, 236);")
        message.setWindowTitle(_("Warning"))
        message.setText(msg)
        message.exec_()

    def dbus_question_massage(self, msg):
        """
        question massage
        :param msg:
        :return:
        """
        reply = QMessageBox.question(None, _("Requestion"), msg, QMessageBox.Yes | QMessageBox.No, QMessageBox.Yes)
        return reply == QMessageBox.Yes

    def dbus_exception_handler(self, msg):
        """
        QMessageBox alert show
        :param alert_type: QMessageBox type
        :param header: alert title
        :param body: alert body info
        :return:
        """
        self.dbus_massage(self.dbus_exception_msg(msg))
        self.cleanup_backend()

    def backend(self):
        """
        This gets initialized lazily.
        :return: D-BUS backend client interface
        """
        if self._dbus_iface is None:
            try:
                bus = dbus.SystemBus()
                self._dbus_iface = dbus.Interface(bus.get_object(DBUS_BUS_NAME, '/DellLinuxAssistant'),
                                                  DBUS_INTERFACE_NAME)
                self._dbus_iface.get_library_info('', '{}')
            except dbus.DBusException as msg:
                return self.dbus_exception_handler(msg)
            except Exception as msg:
                return self.dbus_exception_handler(dbus.DBusException(msg))

        return self._dbus_iface

    def logMetricEvent(self, tag, value):
        """
        server log event
        :param tag: name
        :param value: value
        :return:
        """
        try:
            event = {
                "tag": tag,
                "value": value
            }
            dbus_sync_call_signal_wrapper(self.backend(),
                                          "deal_message",
                                          {},
                                          json.dumps(event))
        except dbus.DBusException as msg:
            self.dbus_exception_handler(msg)

    def logEvent(self, tag):
        """
        server event
        :param tag: msg
        :return:
        """
        try:
            dbus_sync_call_signal_wrapper(self.backend(),
                                          "server_web_message",
                                          {},
                                          tag)
        except dbus.DBusException as msg:
            self.dbus_exception_handler(msg)

    def LogComplexEvent(self, tag, value):
        """
        server complex event
        :param tag:
        :param value: complex value
        :return:
        """
        try:
            dbus_sync_call_signal_wrapper(self.backend(),
                                          "server_web_complex_message",
                                          {},
                                          tag, value)
        except dbus.DBusException as msg:
            self.dbus_exception_handler(msg)

    def LogUsageEvent(self, tag):
        """
        log usage event
        :param tag: usage value
        :return:
        """
        directory = os.path.expanduser('~') + '/' + conf['AppFolderPath']
        config_dir = os.path.join(directory, conf['EventFileName'])
        if not os.path.exists(directory):
            os.makedirs(directory)

        FileHelper().UpdateEventFile({'IS_ACTIVE': tag}, config_dir)


if __name__ == '__main__':

    dbus_handler = DbusHandler()
    dbus_handler.logMetricEvent("TTTTTT", "server_web_complex_message")
