#!/usr/bin/env python3
# -*- coding:utf-8 -*-
"""

This module defines:

    * Event type class which can be supported to send.

    * Functions to generate event file/directory name and queue event.

.. module::DellLinuxAssistant.telemetry.event

    :synopsis: Supported event related class and functions.

------

"""

import logging
import os
import uuid

from datetime import datetime, date

from DellLinuxAssistant.telemetry.file_helper import FileHelper


class Event(object):
    """
    The class is to define the event type data which we can support to send
    """
    def __init__(self, EventTelemetryType, EventType="NULL", EventValue="NULL", utc=None, uuid=None):
        self.EventCreatedUtc = self._GenerateUTC(utc)
        self.EventTag = EventType
        self.EventValue = EventValue
        self.Event_UUID = self._GenerateUUID(uuid)
        self.EventTelemetryType = None
        # verify whether EventTelemetryType is in the given list
        if EventTelemetryType in ('HB', 'OTP', 'LogEvent', 'LogMetricEvent', 'LogFile', 'LogComplexEvent'):
            self.EventTelemetryType = EventTelemetryType
            if EventTelemetryType == 'LogEvent':
                if self.EventTag == 'NULL':
                    raise RuntimeError("LogEvent must have Event Tag value!")
            elif EventTelemetryType in ('LogMetricEvent', 'LogFile', 'LogComplexEvent'):
                if (self.EventTag == "NULL") or (self.EventValue == "NULL"):
                    raise RuntimeError("this type of Event must have both EventTag and EventValue!")
        else:
            raise RuntimeError("the input SoftwareEventType is not valid!")

    def _GenerateUTC(self, utc=None):
        """
        Generate utc timestamp

        :param utc: utc timestamp / None
        :return: utc timestamp
        """
        # if utc timestamp is not assigned, return current utc timestamp
        if not utc:
            return datetime.utcnow()
        else:
            return utc

    def _GenerateUUID(self, UUID=None):
        """
        Generate uuid

        :param UUID: uuid
        :return: uuid
        """
        # if uuid is not assigned, generate and return
        if not UUID:
            return uuid.uuid4().hex
        else:
            return UUID

    def _VerifyAndSaveEvent(self, folderPath, allowedMaxMsgCount, filepath):
        """
        Save event data into specific file.

        :param folderPath: The root path of event files.
        :param allowedMaxMsgCount: The max count of message allowed.
        :param filepath: Event file saved path.
        :type folderPath: string
        :type allowedMaxMsgCount: int
        :type filepath: string
        """
        # verify folder path exist.
        if not os.path.exists(folderPath):
            os.makedirs(folderPath)
        dayDirectories = []
        for _ in os.scandir(folderPath):
            if _.is_dir():
                # get all the "yyyymmdd" timestamp folers ex:1526031684
                dayDirectories.append(os.path.basename(_.path))

        def count_files(dir):
            for root, dirs, files in os.walk(os.path.join(folderPath, dir)):
                return len(files)

        eventFileCount = sum(map(count_files, dayDirectories))
        logging.debug("[VerifyAndSaveEvent] event files is {}".format(eventFileCount))
        # clean up oldest event files if total count of event files is greater than limit.
        if eventFileCount >= allowedMaxMsgCount:
            # count the number of oldest files to be deleted
            file_counts = eventFileCount - allowedMaxMsgCount + 1
            FileHelper().DeleteOldestFile(folderPath, file_counts)
        return FileHelper().SaveData(self, filepath)

    def QueueEvent(self, folderPath, allowedMaxMsgCount=1000):
        """
        Queues an event to be sent

        :param folderPath: The root folder path of events
        :param allowedMaxMsgCount: The max count of message allowed. Default count is 1000.
        :type folderPath: string
        :type allowedMaxMsgCount: int
        :return: Event saved file path
        """
        # verify the root folder path
        if not os.path.exists(folderPath):
            os.makedirs(folderPath)

        # get event date time in UTC
        # build folder path and file name
        dayDirectory = os.path.join(folderPath, self.GenerateEventDateDirectoryName(self.EventCreatedUtc))
        if not os.path.exists(dayDirectory):
            os.makedirs(dayDirectory)
        filePath = os.path.join(dayDirectory, self.GenerateEventTimeFileName(self.EventCreatedUtc, self.Event_UUID))
        # save message file
        if self._VerifyAndSaveEvent(folderPath, allowedMaxMsgCount, filePath):
            return filePath

    def GenerateEventDateDirectoryName(self, CurrentDateTime):
        """
        Generate event directory name by specific datetime

        :param CurrentDateTime: Current datetime timestamp
        :type CurrentDateTime: datetime
        :return: Date directory name
        :rtype: string
        """
        # check the date type is datetime format
        if isinstance(CurrentDateTime, datetime):
            # timestamp
            ts = (CurrentDateTime.date() - date(1970, 1, 1)).total_seconds()
            DateDirectoryName = str(ts).split('.0')[0]
            return DateDirectoryName
        else:
            raise RuntimeError("the input 'CurrentDateTime' type is not valid!")

    def GenerateEventTimeFileName(self, CurrentDateTime, uuid):
        """
        Generate event filename by specific datetime

        :param CurrentDateTime: Current datetime timestamp
        :type CurrentDateTime: datetime
        :param uuid: UUID
        :type uuid: string
        :return: Event filename
        :rtype: string
        """
        # check whether timestamp is in datetime format
        if isinstance(CurrentDateTime, datetime):
            hhmmss = datetime.combine(datetime.min, CurrentDateTime.time()) - datetime.min
            TimeFileName = str(hhmmss.total_seconds()) + '-' + uuid + '.txt'
            return TimeFileName
        else:
            raise RuntimeError("the input 'CurrentDateTime' type is not valid!")


if __name__ == "__main__":
    event = Event('HB')
    # print(event.__class__.__name__)
    # print(event.__module__)
    # event.__class__.__name__ = "test"
    # event.__module__ = "test module"
    # print(event.__class__.__name__)
    # print(event.__module__)
