#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""

This module defines:

    * Functions for getting profile related SW/HW info.

.. module::DellLinuxAssistant.telemetry.profile_helper

    :synopsis: Help functions for profile related info

------

"""

import configparser
import gi
import io
import json
import locale
import os
import sys
import subprocess
import syslog
import platform
import socket

gi.require_version('UDisks', '2.0')
from gi.repository import UDisks

from DellLinuxAssistant.telemetry.telemetry_common import INI_FILE, read_sys_data
from DellLinuxAssistant.telemetry.file_helper import FileHelper

config = configparser.ConfigParser()
config.read(INI_FILE)
conf = config['DEFAULT']


def fetch_output(cmd, data='', environment=os.environ):
    """
    Helper function to just read the output from a command

    :param cmd: Command to execute.
    :param data: Data to be sent to the child process.
    :type cmd: list
    :type data: string
    :return: Output of subprocess result
    :raises: RuntimeError
    """
    # set language to english, avoid nun-english bash return unknown character
    environment['LANG'] = 'en_US'
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE,
                            stdin=subprocess.PIPE,
                            env=environment,
                            universal_newlines=True)
    try:
        (out, err) = proc.communicate(data)
    except UnicodeDecodeError as error:
        if locale.getpreferredencoding() != "UTF-8":
            os.environ["LC_ALL"] = "C.UTF-8"
            locale.setlocale(locale.LC_ALL, '')

        if sys.stdout.encoding != "utf8":
            sys.stdout = io.open(sys.stdout.fileno(), 'w', encoding='utf8')
        if sys.stderr.encoding != "utf8":
            sys.stderr = io.open(sys.stderr.fileno(), 'w', encoding='utf8')
        if sys.stdin.encoding != "utf8":
            sys.stdin = io.open(sys.stdin.fileno(), 'r', encoding='utf8')

        proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                stderr=subprocess.PIPE,
                                stdin=subprocess.PIPE,
                                env=environment,
                                universal_newlines=True)
        (out, err) = proc.communicate(data)

    if proc.returncode is None:
        proc.wait()
    if proc.returncode != 0:
        error = "Command %s failed with stdout/stderr: %s\n%s" % (cmd, out, err)
        syslog.syslog(error)
        raise RuntimeError(error)
    return out


def get_cpu():
    """
    Get the cpu info

    :return: CPU info
    :rtype: string
    """
    cpu_vendor = None
    core_num = 0
    path = "/proc/cpuinfo"
    with open(path, 'r') as rfd:
        for line in rfd.readlines():
            if line.startswith("model name"):
                cpu_vendor = line.split(":")[1]
            if line.startswith("processor"):
                core_num = core_num + 1
    return cpu_vendor.strip()+" x "+str(core_num)


def get_mem():
    """
    Get the memory info

    :return: Memory info
    :rtype: string
    """
    # Amount of memory in the system
    mem = 0
    if os.path.exists('/sys/firmware/memmap'):
        for root, dirs, files in os.walk('/sys/firmware/memmap', topdown=False):
            if os.path.exists(os.path.join(root, 'type')):
                with open(os.path.join(root, 'type')) as rfd:
                    type = rfd.readline().strip('\n')
                if type != "System RAM":
                    continue
                with open(os.path.join(root, 'start')) as rfd:
                    start = int(rfd.readline().strip('\n'), 0)
                with open(os.path.join(root, 'end')) as rfd:
                    end = int(rfd.readline().strip('\n'), 0)
                mem += (end - start + 1)
        mem = float(mem / 1024)
    if mem == 0:
        with open('/proc/meminfo', 'r') as rfd:
            for line in rfd.readlines():
                if line.startswith('MemTotal'):
                    mem = float(line.split()[1].strip())
                    break
    mem = round(mem / 1048575)  # in GB
    return str(mem)+"GB"
    # return mem_total


def get_graphic():
    """
    Get graphic info

    :return: Graphic info
    :rtype: string
    """
    graphic = ''
    num = 0
    for line in fetch_output(['lshw', '-class', 'display']).split('\n'):
        if "vendor" in line:
            num = num + 1
            graphic = graphic + "[graphic " + str(num) + "] " + line.split(":")[1].strip() + '\n'
    return graphic.strip()


def get_disk():
    """
    Get the local machine's disk layout
    the output will be each vendor name and the disk's size

    :return: Disk info
    :rtype: string
    """
    disks = ''
    num = 0
    udisks = UDisks.Client.new_sync(None)
    manager = udisks.get_object_manager()
    drive = None
    for item in manager.get_objects():
        loop = item.get_loop()
        block = item.get_block()
        partition = item.get_partition()
        if loop or \
                partition or \
                not block or \
                not block.get_cached_property("HintPartitionable").get_boolean() or \
                block.get_cached_property("ReadOnly").get_boolean():
            continue

        drive_obj = block.get_cached_property("Drive").get_string()
        if drive_obj == '/':
            continue

        drive = udisks.get_object(drive_obj).get_drive()
        if drive:
            bus = drive.get_cached_property("ConnectionBus").get_string()
            if bus == 'usb':
                continue
            elif bus == 'sdio':
                media = drive.get_cached_property("Media").get_string()
                if not media:
                    continue
        else:
            continue

        devicesize = drive.get_cached_property("Size").unpack()
        if devicesize == 0:
            continue

        devicemodel = drive.get_cached_property("Model").get_string()
        devicesize_gb = "%i" % (devicesize / 1000000000)
        if str(devicesize_gb) + "GB" in devicemodel:
            devicemodel = devicemodel.split(str(devicesize_gb) + "GB")[0].strip()
        num = num + 1
        disks = disks + "[disk " + str(num) + "] " + devicemodel + "(" + devicesize_gb + "GB" + ")" + "\n"

    return disks.strip()

def get_network():
    """
    Get network info

    :return: Network info
    :rtype: string
    """
    Ethernet = ''
    Wireless = ''

    for network in fetch_output(['lshw', '-class', 'network']).split('*-network\n'):
        if "Ethernet interface" in network:
            for line in network.split("\n"):
                if "vendor" in line:
                    Ethernet = Ethernet + line.split(":")[1]
        if "Wireless interface" in network:
            for line in network.split("\n"):
                if "vendor" in line:
                    Wireless = Wireless + line.split(":")[1]


    return Ethernet.strip(), Wireless.strip()

def get_is_netWork():

    Network = 'DISABLED'
    try:
        connect = fetch_output(['ping', '8.8.8.8', '-c', '1'])
        if connect:
            Network = 'ABLED'
    except:
        Network = 'DISABLED'

    return Network

def get_bios_version():
    """
    Get bios version info

    :return: BIOS info
    :rtype: string
    """
    path = '/sys/class/dmi/id/bios_version'
    return read_sys_data(path).decode()


def get_productname():
    """
    Get product name

    :return: Product name
    :rtype: string
    """
    path = '/sys/class/dmi/id/product_name'
    return read_sys_data(path).decode()

def get_lob():
    """
    Get the matched BIOS item Lob info

    :return: BIOS
    :rtype: string
    """
    path = '/sys/class/dmi/id/product_family'
    return read_sys_data(path).decode()

def get_SN():
    """
    Get the matched BIOS item SN info

    :return: SN info
    :rtype: string
    """
    # SN need root privillege
    path = os.path.join(conf['PlatinumFolderPath'], conf['Platinum_Setting_Filename'])
    if os.path.exists(path):
        with open(path, 'r') as input:
            data = json.load(input)
        if data['SN']:
            return data['SN']
    else:
        path = os.path.join('/sys/class/dmi/id', "product_serial")
        return read_sys_data(path).decode()


def get_UUID():
    """
    Get the matched BIOS item UUID info

    :return: UUID info
    :rtype: string
    """
    path = os.path.join(conf['PlatinumFolderPath'], conf['Platinum_Setting_Filename'])
    if os.path.exists(path):
        with open(path, 'r') as input:
            data = json.load(input)
        if data['UUID']:
            return data['UUID']
    else:
        path = os.path.join('/sys/class/dmi/id', "product_uuid")
        return read_sys_data(path).decode()


def get_kernel_version():
    """
    Get the kernel version to check in future

    :return: kernel version info
    :rtype: string
    """
    path = "/proc/version"
    return read_sys_data(path).split()[2].decode()

def get_log_event_switch():
    """
    get switch status True:on False:off
    :return: True or False
    """
    config_dir = os.path.join(os.path.expanduser('~'), conf['AppFolderPath'], conf['EventFileName'])
    data = FileHelper().GetDataFromFile(config_dir, True)

    return data.get('IS_ACTIVE') if data else True

def get_system():
    """
    get os system
    :return:
    """
    return platform.platform()

def get_osLanguage():
    """
    get os language
    :return:
    """
    val = os.popen("echo $LANG")
    # print(type(val), val)
    return val.read()

def get_OOBE():
    """
    OOBE boot start time
    :return: time
    """
    path = os.path.join(conf['PlatinumFolderPath'], conf['Platinum_Setting_Filename'])
    data = FileHelper().GetDataFromFile(path, True)
    return data.get('OOBE')

if __name__ == "__main__":
    # import time
    # print(time.time())
    # print(get_cpu())
    # print(get_disk())
    # print(get_graphic())
    # print(time.time())
    #
    # print(get_network())
    # print(time.time())

    # print(get_log_event_switch())
    print(update_log_event_switch(False))
