#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""

This module defines:

    * Client class for sending message through scheduler or directory.

.. module::DellLinuxAssistant.telemetry.queue_message_client

    :synopsis: Message sending client related class.

------

"""

import logging
from datetime import datetime, timedelta

from DellLinuxAssistant.telemetry.event import Event
from DellLinuxAssistant.telemetry.message_helper import MessageHelper
from DellLinuxAssistant.telemetry.telemetry_common import check_log_switch


class QueueMessageClient(object):
    def __init__(self, configure, scheduler):
        """
        :param configure: configure is from backend service's conf file,
        "param scheduler: scheduler is the scheduler's handler from backed daemon service
        """
        self.configure = configure
        self.scheduler = scheduler

    def triggerSend(self, filePath, header, Schedule=True):
        """
        This method is to send the event through schedule configuration

        :param filePath: Path of event files.
        :param header: Header used for SendEvent.
        :param Schedule: Scheduler used for setting ways to send event
        :type header: list
        """
        # if through scheduler
        if Schedule:
            if Schedule == "wait":
                # it will trigger the send event 5 minutes later
                logging.info("[triggerSend]later scheduler,application uses,{}".format(self.scheduler))
                job = self.scheduler.add_interval_job(MessageHelper(self.configure).SendEvent, minutes=5, args=[filePath, header], max_runs=1)
            else:
                # it will trigger the event nearly immediately
                job = self.scheduler.add_interval_job(MessageHelper(self.configure).SendEvent, minutes=1, start_date=datetime.now()+timedelta(seconds=1), args=[filePath, header], max_runs=1)
                logging.info("[triggerSend]this event is added on the scheduler,running right now,{}".format(self.scheduler))
            # return job
        else:
            # if not through scheduler, the event will be sent directly which need to block the process.
            # the scenario always happens at stopping service event or uninstalling service event.
            try:
                MessageHelper(self.configure).SendEvent(filePath)
                logging.info("[triggerSend]Send Event without Scheduler")
            except Exception as e:
                logging.error('[triggerSend]call SendEvent failed,ERROR:{}'.format(str(e)))

    @check_log_switch
    def Heartbeat(self):
        """
        queue heartbeat event in local
        """
        filePath = self.QueueSoftWareEvent('HB')
        logging.info("sending HB ...")
        header = ['HB']
        self.scheduler.add_interval_job(MessageHelper(self.configure).SendEvent, weeks=1, start_date=datetime.now() + timedelta(seconds=1), args=[filePath, header])
        logging.info("[Heartbeat]HB event is added on the scheduler,running right now and in every week,{}".format(self.scheduler))

    @check_log_switch
    def OneTimePing(self, Schedule=True):
        """
        OTP event which is send only once at the boot up time
        :param Schedule: Scheduler used for setting ways to send event
        """
        filePath = self.QueueSoftWareEvent('OTP')
        logging.info("sending OTP ...")
        header = ['OTP']
        return self.triggerSend(filePath, header, Schedule)

    @check_log_switch
    def LogEvent(self, EventTag, Schedule=True):
        """
        Schedule variable is to determine whether the event will be sent through schedule thread or not,default is through schedule
        """
        filePath = self.QueueSoftWareEvent('LogEvent', EventTag)
        logging.info("sending LogEvent ... ET:{}".format(EventTag))
        header = ['LogEvent', EventTag]
        return self.triggerSend(filePath, header, Schedule)

    @check_log_switch
    def LogLogMetricEvent(self, EventTag, EventValue, Schedule=True):
        """
        Log Metric Event
        :param EventTag: event tag
        :param EventValue: event value
        :param Schedule: Schedule
        :return:
        """
        filePath = self.QueueSoftWareEvent('LogMetricEvent', EventTag, EventValue)
        logging.info("sending LogLogMetricEvent ... ET:{},EV:{}".format(EventTag, EventValue))
        header = ['LogMetricEvent', EventTag, EventValue]
        return self.triggerSend(filePath, header, Schedule)

    @check_log_switch
    def LogComplexEvent(self, EventTag, EventValue, Schedule=True):
        """
        Log Complex Event
        :param EventTag:
        :param EventValue:
        :param Schedule:
        :return:
        """
        filePath = self.QueueSoftWareEvent('LogComplexEvent', EventTag, EventValue)
        logging.info("sending LogComplexEvent ... ET:{},EV:{}".format(EventTag, EventValue))
        header = ['LogComplexEvent', EventTag, EventValue]
        return self.triggerSend(filePath, header, Schedule)

    @check_log_switch
    def LogFile(self, EventTag, EventValue, Schedule=True):
        filePath = self.QueueSoftWareEvent('LogFile', EventTag, EventValue)
        logging.info("sending LogFile ... ET:{},EV:{}".format(EventTag, EventValue))
        header = ['LogFile', EventTag, EventValue]
        return self.triggerSend(filePath, header, Schedule)

    def QueueSoftWareEvent(self, EventTelemetryType, *args):
        """
        It is the shared interface to send all kinds of events
        EventTelemetryType is valid in one of 'HB','OTP','LogEvent' etc.
        `**kwargs provides` 'ET=' or 'EV='
        """
        event = Event(EventTelemetryType, *args)
        # support debug event for server to filter data
        if self.configure["debug"] == 'yes':
            if event.EventTelemetryType == "HB" or event.EventTelemetryType == "OTP":
                event.EventTag = event.EventTelemetryType + "(DEBUG)"
                event.EventTelemetryType = "LogEvent"
            else:
                event.EventTag = event.EventTag + "(DEBUG)"
        filePath = event.QueueEvent(self.configure['PlatinumFolderPath'], allowedMaxMsgCount=int(self.configure['allowedMaxMsgCount']))
        return filePath


if __name__ == '__main__':
    from ApschedulerStandalone.scheduler import Scheduler, SchedulerAlreadyRunningError

    try:
        scheduler = Scheduler()
        scheduler.start()
        client = QueueMessageClient(Configuration(), scheduler)
        # client = QueueMessageClient(Configuration())
        client.QueueSoftWareEvent('HB')
        client.QueueSoftWareEvent('OTP')
        # client.Heartbeat()
        scheduler.shutdown()
    except SchedulerAlreadyRunningError:
        pass
