#!/usr/bin/python3
# -*- coding: utf-8 -*-
"""

This module defines:

    + Decorators to sent running time of event process.

    + Common functions for setting privileges, setting up logging, checking connections and package version.

.. module::DellLinuxAssistant.telemetry.telemetry_common

    :synopsis: Common functions and decorators.

------

"""

import functools
import grp
import pwd
import logging
import multiprocessing
import os
import socket
import sys
import collections

from gi.repository import GLib
from systemd.journal import JournalHandler
from threading import Thread

# Translation Support
DOMAIN = 'dell-telemetry'
LOCALEDIR = '/usr/share/locale'
INI_FILE = '/etc/dell-linux-assistant/DellLinuxAssistant.ini'


# internal decorator
class TaskletTimeoutError(Exception):
    """Internal decorator"""
    pass


class tasklet(object):
    """
    This class is used as decorator which to limit sent event process's running time.
    If given time is out, the process still doesn't fininshed,
    it will force end up the process in order not to block following actions
    """
    def __init__(self, timeout=10, maxretry=1, result=True):
        """
        Initialize timeout and maxretry count.
        Default values will be timeout after 10 seconds and maxretry for only once
        """
        self.timeout = timeout
        self.maxretry = maxretry
        self.result = result

    def __call__(self, original_func):
        decorator_self = self

        @functools.wraps(original_func)
        def wrapper(*args, **kwargs):
            count = decorator_self.maxretry
            q = multiprocessing.Queue()
            q.put(None)
            real_args = args
            real_kwargs = kwargs

            def __run(func, real_args, real_kwargs):
                result = original_func(*real_args, **real_kwargs)
                q.get()
                q.put(result)
            # retry for specific times
            while count > 0:
                proc_args = (original_func, real_args, real_kwargs)
                proc_task = multiprocessing.Process(target=__run, args=proc_args)
                proc_task.start()

                if decorator_self.result:

                    if decorator_self.timeout > 0:
                        # still timeout over then process terminate and main process stop
                        proc_task.join(decorator_self.timeout)
                        if proc_task.is_alive():
                            proc_task.terminate()
                            count -= 1
                            continue
                    else:
                        # still process finish and main process stop
                        proc_task.join()
                        count -= 1

                    result = q.get()
                    return result
                else:
                    # no result so child run and main process also go run
                    count -= 1

            if decorator_self.result:
                try:
                    raise TaskletTimeoutError(u'timeout : %d seconds X %d retry times.' %
                                              (decorator_self.timeout, decorator_self.maxretry))
                except TaskletTimeoutError as e:
                    logging.warning(str(e))
        return wrapper


def dbus_sync_call_signal_wrapper(dbus_iface, func, handler_map, *args, **kwargs):
    """
    Run a D-BUS method call while receiving signals.
    This function is an Ugly Hack™, since a normal synchronous dbus_iface.fn()
    call does not cause signals to be received until the method returns. Thus
    it calls func asynchronously and sets up a temporary main loop to receive
    signals and call their handlers; these are assigned in handler_map (signal
    name → signal handler).
    """
    if not hasattr(dbus_iface, 'connect_to_signal'):
        # not a D-BUS object
        return getattr(dbus_iface, func)(*args, **kwargs)

    def _h_reply(*args, **kwargs):
        """protected method to send a reply"""
        global _h_reply_result
        _h_reply_result = args
        loop.quit()

    def _h_error(exception=None):
        """protected method to send an error"""
        global _h_exception_exc
        _h_exception_exc = exception
        loop.quit()

    loop = GLib.MainLoop()
    global _h_reply_result, _h_exception_exc
    _h_reply_result = None
    _h_exception_exc = None
    kwargs['reply_handler'] = _h_reply
    kwargs['error_handler'] = _h_error
    kwargs['timeout'] = 86400
    for signame, sighandler in handler_map.items():
        dbus_iface.connect_to_signal(signame, sighandler)
    dbus_iface.get_dbus_method(func)(*args, **kwargs)
    loop.run()
    if _h_exception_exc:
        raise _h_exception_exc
    return _h_reply_result


def drop_privileges(uid_name='nobody', gid_name='nogroup'):
    """
    Drop privileges for specific user & group.
    Do nothing if current user is root

    :param uid_name: Specific user id name.
    :param gid_name: Specific group id name.
    :type uid_name: string
    :type gid_name: string
    """
    starting_uid = os.getuid()
    starting_gid = os.getgid()

    starting_uid_name = pwd.getpwuid(starting_uid)[0]

    logging.info('drop_privileges: started as %s/%s' %
                 (pwd.getpwuid(starting_uid)[0],
                     grp.getgrgid(starting_gid)[0]))

    # uid for root shall be 0
    if starting_uid != 0:
        # If we're not root user, just log and return
        logging.info("drop_privileges: already running as '%s';" %
                     starting_uid_name)
        return

    # If we started as root, drop privs and become the specified user/group
    if starting_uid == 0:

        # Get the uid/gid from the name
        running_uid = pwd.getpwnam(uid_name)[2]
        running_gid = grp.getgrnam(gid_name)[2]

        # Try setting the new uid/gid
        try:
            os.setgid(running_gid)
        except OSError as e:
            logging.error('Could not set effective group id: %s' % e)

        try:
            os.setuid(running_uid)
        except OSError as e:
            logging.error('Could not set effective user id: %s' % e)

        # Ensure a very convervative umask
        new_umask = 0o77
        old_umask = os.umask(new_umask)
        logging.info('drop_privileges: Old umask: %s, new umask: %s' %
                     (oct(old_umask), oct(new_umask)))

    final_uid = os.getuid()
    final_gid = os.getgid()
    logging.info('drop_privileges: running as %s[%s]/%s[%s]' %
                 (pwd.getpwuid(final_uid)[0], str(final_uid),
                     grp.getgrgid(final_gid)[0], str(final_gid)))


def is_connected(hostname):
    """
    Verify whether hostname is reachable

    :param hostname: Specific hostname.
    :type hostname: string
    :return: `True` if reachable, `False` if not reachable
    :rtype: bool
    """
    try:
        # see if we can resolve the host name -- tells us if there is
        # a DNS listening
        host = socket.gethostbyname(hostname)
        # connect to the host -- tells us if the host is actually
        # reachable
        socket.create_connection((host, 80), 2)
        return True
    except Exception:
        pass
    return False


def setup_logging(debug=False):
    """
    Setup logging.

    :param debug: Used for setting logging level. (Default is False)
    :type debug: bool
    """
    logging.raiseExceptions = False
    if debug:
        logging.basicConfig(level=logging.DEBUG,
                            format='%(asctime)s %(levelname)s: %(message)s')
    else:
        logging.basicConfig(level=logging.WARNING,
                            format='%(levelname)s: %(message)s')
    logger = logging.getLogger()
    logger.addHandler(JournalHandler())


def check_version(package='dell-linux-assistant'):
    """
    Queries the package management system for the current tool version

    :param package: Package name for query.
    :type package: string
    :return: package version / 'unknown'
    :rtype: string
    """
    try:
        import apt.cache
        cache = apt.cache.Cache()
        if cache[package].is_installed:
            return cache[package].installed.version
    except Exception as msg:
        print("Error checking %s version: %s" % (package, msg), file=sys.stderr)
        return "unknown"


def read_sys_data(path):
    """
    Read data from specific file.

    :param path: Specific file path.
    :type path: string
    :return: data in the file
    :rtype: string
    """
    if not os.path.exists(path):
        return None
    else:
        try:
            with open(path, 'rb') as rfd:
                value = rfd.read().strip()
                return value
        except Exception as e:
            logging.error("[read_sys_data]error is {}".format(str(e)))


def check_log_switch(func):
    """
    decorator check switch status: True write logfile, False no any operation
    :param func: callback function
    :return: func
    """
    @functools.wraps(func)
    def wrapper(*args, **kw):

        from DellLinuxAssistant.telemetry import profile_helper
        event_status = profile_helper.get_log_event_switch()
        if event_status:
            logging.info("[log event open]status: {}, func.name: {}".format(event_status, func.__name__))
            return func(*args, **kw)
        else:
            logging.info("[log event close]status: {}, func.name: {}".format(event_status, func.__name__))
            return None
    return wrapper


quese = collections.deque()
class ObjectThread(object):

    def __init__(self, result=True):
        self.result = result

    def __call__(self, original_func):
        decorator_self = self

        @functools.wraps(original_func)
        def wrapper(*args, **kwargs):

            quese.append(None)
            real_args = args
            real_kwargs = kwargs
            def __run(func, real_args, real_kwargs):
                res = original_func(*real_args, **real_kwargs)
                quese.append(res)

            proc_args = (original_func, real_args, real_kwargs)
            proc_task = Thread(target=__run, args=proc_args)
            proc_task.start()

            if decorator_self.result:

                # still process finish and main process stop
                proc_task.join()
                return quese.pop()

        return wrapper


class Struct:
    """ Convert nested Python dict to object """
    def __init__(self, **entries):
        self.__dict__.update(entries)

import time
@tasklet(timeout=10, maxretry=1)
def test():
    time.sleep(10)
    print("start3 = ", time.time())
    return "test"

@tasklet(timeout=5, maxretry=1)
def bug():
    time.sleep(10)
    print("start2 = ", time.time())
    return "bug"


if __name__ == '__main__':
    import time
    REMOTE_SERVER = "connectivity-check.ubuntu.com"
    start = time.time()
    print(is_connected(REMOTE_SERVER))
    end = time.time()
    print(end-start)
